/*
 *  File: overlay.js
 *  Description: httProxy oevrlay event handling
 *  Author: Ovais Khan
 *
 *  Portions based on GPLed code by Ted Mielczarek
 * 
 */

// constants
const GT_URL = "http://www.google.com/translate?langpair=en|en&u=";
const ANON_STYLE_URL = "A";
const CDN_STYLE_URL = "C";
const PKBLOG_STYLE_URL = "P";
const PROXY_SEP = "##~##";
const DET_SEP = "#~#";
const MAX_PROXIES = 50;

// globals
var httproxyBundle; // string-bundle-httproxy
var proxyNames = new Array(MAX_PROXIES)
var proxyURLs = new Array(MAX_PROXIES)
var proxyStyles = new Array(MAX_PROXIES)
var numProxies = 0
var proxyMenuListModified = true;
var proxyListModified = true;
var newProxyList = "";

// httProxy logic
var httproxy = {

  menuItems: new Array("httproxy-view", "httproxy-viewNewTab"),
  linkMenuItems: new Array("httproxy-viewlink", "httproxy-viewlinkNewTab"),
  
  readOptions: function()
  {
    var prefservice = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService);
    var prefs = prefservice.getBranch("");
    var proxies = null;
    var proxyList = null;
    var proxyDetails = null;
    var defaultProxy = this.getDefaultProxy();
  
    if (!proxyListModified)
      return;
    
    proxies = prefs.getCharPref("extensions.httproxy.proxies");
    
    // separate proxies
    proxyList = proxies ? proxies.split(PROXY_SEP) : "";
    
    // separate proxy names and other info
    for (var i = 0; i < proxyList.length && i < MAX_PROXIES; i++) {
       if (proxyList[i] != '') {
         proxyDetails = proxyList[i] ? proxyList[i].split(DET_SEP) : "";
         proxyNames[i] = proxyDetails[0];
         proxyURLs[i] = proxyDetails[1];
         proxyStyles[i] = proxyDetails[2];
       }
    }
    numProxies = i;
    proxyMenuListModified = true;
    
    //Components.utils.reportError("Read Options; numProxies: " + numProxies); 
  },
  
  getDefaultProxy: function()
  {
    var defaultProxy = -1;
  
    var prefservice = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService);
    var prefs = prefservice.getBranch("");
    if (prefs.getPrefType("extensions.httproxy.defaultproxy") == prefs.PREF_INT)
      defaultProxy = prefs.getIntPref("extensions.httproxy.defaultproxy");
      
    return defaultProxy;
  },
  
  startsWith: function(st, pref)
  {
    return( (pref.length > 0) && (st.substring(0, pref.length) == pref) );
      
  },
  
  checkUrl: function(url)
  {
    return ( typeof(url) != 'undefined'
      && url.substr
      && this.startsWith(url, 'http://') );
  },
  
  proxifyURL: function(url, proxyIndex) 
  {
   if (proxyIndex == null)
     proxyIndex = this.getDefaultProxy();
     
   if (this.checkUrl(url))
   {
     // convert to proxy url
     var proxyURL = proxyURLs[proxyIndex];
     var newUrl = null;
     
     // is this a CDN style proxy
     if (proxyStyles[proxyIndex] == ANON_STYLE_URL) {
       newUrl = proxyURLs[proxyIndex] + url;
     }
     else if (proxyStyles[proxyIndex] == CDN_STYLE_URL) {
        var myRegEx = new RegExp("^([^:]+:)?/{2,}([^:/]+)(:\[0-9]+)?(/(.*))?");
        var regExRes = myRegEx.exec(url);
        if (regExRes == null || regExRes[1] != "http:")
            return "error";
            
        newUrl = "http://" + regExRes[2]
        // append CDN proxy name as required
        if (regExRes[3] != null)   // there is a port part
            newUrl = newUrl + "." + (new String(regExRes[3]).substr(1));
        newUrl = newUrl + proxyURLs[proxyIndex] + ((regExRes[5] == null) ? "" : regExRes[5]);
     }
     else if (proxyStyles[proxyIndex] == PKBLOG_STYLE_URL) {
        var myRegEx = new RegExp("^([^:]+:)?/{2,}([^:/.]+)(\.blogspot\.com)(:\[0-9]+)?(/.*)?");
        var regExRes = myRegEx.exec(url);
        if (regExRes == null || regExRes[1] != "http:")
            return "error";
        newUrl = proxyURLs[proxyIndex] + regExRes[2] + ((regExRes[5] == null) ? "" : regExRes[5])
     }
     else
       return "error";
     
     return newUrl;
   }
   else
     return "error";
  },
  
  isEnableForceLoad: function() 
  {
    var bEnabled = true;
  
    var prefservice = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService);
    var prefs = prefservice.getBranch("");
    if (prefs.getPrefType("extensions.httproxy.defaultproxy") == prefs.PREF_INT)
      bEnabled = prefs.getBoolPref("extensions.httproxy.enableforceload");
      
    return bEnabled;
  },

  isEnableBSForceLoad: function() 
  {
    var bEnabled = true;
  
    var prefservice = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService);
    var prefs = prefservice.getBranch("");
    if (prefs.getPrefType("extensions.httproxy.defaultproxy") == prefs.PREF_INT)
      bEnabled = prefs.getBoolPref("extensions.httproxy.enableforceblogspot");
      
    return bEnabled;
  },

  checkForceLoad: function(url) 
  {
    // for testing redirect all blogspot blogs
    var bTest = false;
    
    if (this.isEnableBSForceLoad())
      bTest = bTest || this.isBlogSpotURL(url);
  
// for testing only  
//    if (url == "http://www.yahoo.com/")
//      bTest = bTest || true;
      
    return bTest;
  },

  isBlogSpotURL: function(url) 
  {
    // check if url is in one of the required format
    var myRegEx = new RegExp("^([^:]+:)?/{2,}([^:/.]+)(\.blogspot\.com)(:\[0-9]+)?(/.*)?");
    var regExRes = myRegEx.exec(url);
    if (regExRes == null || regExRes[1] != "http:")
      return false;
    
    // check if url is not already in CDN style
    myRegEx = new RegExp("^([^:]+:)?/{2,}([^:/.]+)(\.blogspot\.com)(.[^:/]+)(:\[0-9]+)?(/.*)?");
    regExRes = myRegEx.exec(url);
    if (regExRes != null) // if this test fails, then this is not a CDN style url
      return false;
      
    return true;
  },

  //	check whether the document behind a dom-loaded event is framed
  isFramed: function(aEvent) 
  {
    var framed = aEvent.originalTarget && 
      window._content && window._content.document &&
      (window._content.document != aEvent.originalTarget);
  
    return(framed);
  },

  openURL: function(url, newTab) 
  {
    if (newTab)
      gBrowser.addTab(url);
    else
      window._content.location = url;
  },
  
  proxyViewURL: function() 
  {
    var url = document.getElementById('urlbar').value;
    url = this.proxifyURL (url);
    if (url != "error")
      this.openURL(url, false);
    else
      alert (httproxyBundle.getString("httproxy.InvalidURL")); //Invalid URL to be proxied
  },

  proxyViewURLUsingProxy: function(oMenuItem) 
  {
    var index = oMenuItem.getAttribute("index");
    // idnetify and use proxy server properly
    if (index < 0 || index > numProxies)
      return false;
    
    var url = document.getElementById('urlbar').value;
    url = this.proxifyURL (url, index);
    if (url != "error") {
      this.openURL(url, false);
      return true;
    }
    else {
      alert (httproxyBundle.getString("httproxy.InvalidURL")); //Invalid URL to be proxied
      return false;
    }
  },
  
  proxyViewPage: function(newTab) 
  {
    var url = gBrowser.currentURI.spec;
    url = this.proxifyURL (url);
    if (url != "error")
      this.openURL(url, newTab);
    else
      alert (httproxyBundle.getString("httproxy.InvalidURL")); //Invalid URL to be proxied
  },
  
  proxyViewLink: function(newTab) 
  {
    if (gContextMenu)
    {
       var url = this.contextLinkURL();
       url = this.proxifyURL (url);
       if (url != "error")
         this.openURL(url, newTab);
       else
         alert (httproxyBundle.getString("httproxy.InvalidURL")); //Invalid URL to be proxied
    }
  },
  
  contextLinkURL: function()
  {
    if (gContextMenu && gContextMenu.getLinkURL)
    {
      return(gContextMenu.getLinkURL());
    }
    
    return(false);
  },
  
  addNewProxy: function()
  {
    var proxyList = document.getElementById('proxylist');
    var newName = document.getElementById('newName').value;
    var style = document.getElementById('style').selectedItem.value;
    var newUrl = document.getElementById('newUrl').value;
    var error = false;
    
    if (style == "")
      error = true;
    else if (style != CDN_STYLE_URL && !this.startsWith(newUrl, 'http://'))
      error = true;
    else if (newName == "")
      error = true;
    
    if (error) {
      alert (httproxyBundle.getString("httproxy.InvalidProxy")); //Invalid Proxy Data
      return;
    }
    
    if (proxyList.childNodes.length >= MAX_PROXIES) {
      alert (httproxyBundle.getString("httproxy.MaxProxies")); 
      //Sorry, maximum proxy limit has reached. No proxy can be added. This is a limitation with current version
      return;
    }
    
    newProxyList = newProxyList + PROXY_SEP + newName + DET_SEP + newUrl + DET_SEP + style;
    proxyList.appendItem(newName + " - " + newUrl);
    proxyListModified = true;
  },
    
  readSettingsOptions: function()
  {
    var proxyList = document.getElementById('proxylist');
    this.readOptions();
    
    for (var i = 0; i < numProxies; i++) {
      proxyList.appendItem(proxyNames[i] + " - " + proxyURLs[i]);
    }
    
    proxyList.clearSelection();
    proxyList.selectedIndex = this.getDefaultProxy();
    
    var autoProxy = document.getElementById('enable-auto-proxy');
    var autoBlogSpot = document.getElementById('enable-auto-blogspot');
    
    autoProxy.setAttribute("checked", this.isEnableForceLoad());
    autoBlogSpot.setAttribute("checked", this.isEnableBSForceLoad());
  },
  
  setSettingsOptions: function()
  {
    var prefService = Components.classes["@mozilla.org/preferences-service;1"]
                         .getService(Components.interfaces.nsIPrefService);
    var prefs = prefService.getBranch("");
    var autoProxy = document.getElementById('enable-auto-proxy');
    var autoBlogSpot = document.getElementById('enable-auto-blogspot');
    var proxyList = document.getElementById('proxylist');
    var selected = proxyList.selectedIndex;
    var proxies = prefs.getCharPref("extensions.httproxy.proxies");
  
    if (selected < 0)
    {
      alert (httproxyBundle.getString("httproxy.InvalidSelection"));  // Invalid Selection
      return;
    }
  
    // Option to add user-defined proxies
    prefs.setCharPref("extensions.httproxy.proxies", proxies + newProxyList);
    prefs.setIntPref("extensions.httproxy.defaultproxy", selected);
    // set the site related preferences
    prefs.setBoolPref("extensions.httproxy.enableforceload", autoProxy.checked);
    prefs.setBoolPref("extensions.httproxy.enableforceblogspot", autoBlogSpot.checked);
    // after change in settings, redraw the settings menu
    proxyListModified = true;
    
    // update proxy options - for any change in proxies
    this.readOptions();
    window.close();
  },

  setDefaultProxy: function (oMenuItem)
  {
    var index = oMenuItem.getAttribute("index");
    // idnetify and use proxy server properly
    if (index < 0 || index > numProxies)
      return false;

    var prefService = Components.classes["@mozilla.org/preferences-service;1"]
                         .getService(Components.interfaces.nsIPrefService);
    var prefs = prefService.getBranch("");
    prefs.setIntPref("extensions.httproxy.defaultproxy", index);
    // after change in settings, redraw the settings menu
    proxyMenuListModified = true;
    
    return true;
  },
   
  clearProxyList: function(aParent)
  {
	if(aParent!=null){
		var aProxyListElem = aParent.getElementsByAttribute("class", "proxy-name");
		for(var c = aProxyListElem.length-1; c >= 0 ; c--){
          if (aProxyListElem[c].getAttribute("parent") == aParent.id) {
            aParent.removeChild(aProxyListElem[c]);
          }
		}
	}
    return true;
  },
   
  fillProxyList: function(aParent)
  {
    var oItem = null;
    var command;

    this.readOptions();

    //Components.utils.reportError("fillProxyList Options; numProxies: " + numProxies 
      //+ " proxyMenuListModified: " + proxyMenuListModified + " parent: " + aParent.id); 

    if (aParent.id == "httpproxy-popup-default-proxy")
      command = " httproxy.setDefaultProxy(this); event.preventBubble();";
    else if (aParent.id == "httproxy-toolbar-menu")
      command = " httproxy.proxyViewURLUsingProxy(this);  event.preventBubble();";
   
    // Remove old entries if any
    this.clearProxyList(aParent);
    
    for (var i = 0; i < numProxies; i++) {
      // create menu item
      oItem = document.createElement("menuitem");
      oItem.setAttribute("id", aParent.id + "-proxy-" + i);
      oItem.setAttribute("parent", aParent.id);
      oItem.setAttribute("label", proxyNames[i]);
      oItem.setAttribute("tooltiptext", proxyURLs[i]);
      oItem.setAttribute("type", "checkbox");
      oItem.setAttribute("index", i);
      oItem.setAttribute("proxyStyle", proxyStyles[i]);
      oItem.setAttribute("class", "proxy-name");
      oItem.setAttribute("oncommand", command);

      if (i == this.getDefaultProxy()){
        //createMenuItem(aParent, i, "** " + proxyNames[i]);
        oItem.setAttribute("checked", true);
      }
      else {
        //createMenuItem(aParent, i, proxyNames[i]);
        oItem.setAttribute("checked", false);
      }
      // append this menu item to the menu
      aParent.appendChild(oItem);
    }
    
    proxyMenuListModified = false;
    return true;
  },
   

  httproxyContext: function(aEvent)
  {
    if (gContextMenu)
    {
       var i;
       var menuitem = null;
       
       var anyMenuItemHidden =
         gContextMenu.isTextSelected || // neglect selected text
         gContextMenu.onImage ||        // neglect images
         gContextMenu.onTextInput;      // neglect text input boxes
       var menuItemHidden = 
         gContextMenu.onLink ||         // neglect links
         anyMenuItemHidden ||           
         !this.checkUrl(gBrowser.currentURI.spec); // check URL
       var linkMenuItemHidden =
         !gContextMenu.onLink ||        // only show if link
         anyMenuItemHidden ||
         !this.checkUrl(this.contextLinkURL());   // check Link URL
       
       //Components.utils.reportError("anyMenuItemHidden: " + anyMenuItemHidden);
       //Components.utils.reportError("menuItemHidden: " + menuItemHidden);
       //Components.utils.reportError("linkMenuItemHidden: " + linkMenuItemHidden);
       
       for ( i=0; i < this.menuItems.length; i++)
       {
          menuitem = document.getElementById(this.menuItems[i]);
          if (menuitem)
             menuitem.hidden = menuItemHidden;
       }
       for ( i=0; i < this.linkMenuItems.length; i++)
       {
          menuitem = document.getElementById(this.linkMenuItems[i]);
          if (menuitem)
             menuitem.hidden = linkMenuItemHidden;
       }
    }
  },
  
  onLoad: function() {
    //Components.utils.reportError("httproxy.onload: " + this);
  
    var menu = document.getElementById("contentAreaContextMenu");
    if (menu)
    {
       menu.addEventListener("popupshowing", httproxyContextListener,false);
    }
    // get string bundle for message strings
    httproxyBundle = document.getElementById("string-bundle-httproxy");
    // proxy list is being reloaded, set appropriate flag so later functions can reread proxy
    proxyListModified = true;
    // read proxy options
    this.readOptions();
    // initialization code
    this.initialized = true;
    this.httProxyInitialShow();
    return true;
  },
   
  // show custom buttons only in the initial startup
  httProxyInitialShow: function() {
      var bar = document.getElementById('nav-bar');
      if (bar && bar.currentSet) {
          var prefService = Components.classes["@mozilla.org/preferences-service;1"]
                             .getService(Components.interfaces.nsIPrefService);
          var prefs = prefService.getBranch("");
          
          var currentset = bar.currentSet;
          var buttons = currentset.replace(/__empty/, '').split(',');
  
          if (!prefs.getBoolPref('extensions.httproxy.initialshow.buttons')) {
              if (currentset.indexOf('httproxy-button') < 0) {
                  if (currentset.indexOf('go-container') < 0) {
                      buttons.push('httproxy-button');
                  }
                  else {
                      currentset = currentset.replace(/go-container/, 'go-container,httproxy-button');
                      buttons = currentset.split(',');
                  }
              }
              prefs.setBoolPref('extensions.httproxy.initialshow.buttons', true);
          }
          currentset = bar.currentSet.replace(/__empty/, '');
          var newset = buttons.join(',');
          if (currentset != newset) {
              bar.currentSet = newset;
              bar.setAttribute('currentset', newset);
              document.persist(bar.id, 'currentset');
          }
      }
  },
  
  checkForced: function(aEvent) {
    var i;
    var doc = aEvent.originalTarget;
    
//    Components.utils.reportError("checkForced; url: " + doc.location.href
//      + "; isEnableForceLoad: " + this.isEnableForceLoad()
//      + "; isFramed: " + this.isFramed(aEvent));

    // check if forced loading using cgi proxy is enabled or not
    if (! this.isEnableForceLoad())
      return;
    
    // we only want to reload the topmost frame, 
    // for remaining, user needs to open frame in a tab or window
    if (this.isFramed(aEvent))
      return;
    
    if (doc.location && doc.location.href && this.checkForceLoad(doc.location.href)) {
      // 
      var ws = gBrowser.docShell.QueryInterface(Components.interfaces.nsIRefreshURI);
    
      if (ws)
        ws.cancelRefreshURITimers();
        
      var url = this.proxifyURL (doc.location.href);
      if (url != "error")
        doc.location.href = url;
      else
        Components.utils.reportError(httproxyBundle.getString("httproxy.InvalidURL") + "; URL: " 
          + e.originalTarget.location.href); //Invalid URL to be proxied
    }
  }
  
};

function httproxyContextListener(e)
{
  return(httproxy.httproxyContext(e));
}

var gHttpProxyInitialized = false;

function httproxyInitListener(e)
{
  if (gHttpProxyInitialized) return false;
  gHttpProxyInitialized = true;
  return(httproxy.onLoad(e));
}

function httproxyContentLoadedListener(e)
{
//  Components.utils.reportError("httproxyContentLoadedListener: " + e.originalTarget.location.href);
  httproxy.checkForced(e);
}

function httproxyPageShowListener(e)
{
/*
  Components.utils.reportError("httproxyPageShowListener: " + e.originalTarget.location.href
    + "e.persisted " + e.persisted);
  if (e.persisted)
    httproxy.checkForced(e);
*/    
}

function readSettingsOptions()
{
  return (httproxy.readSettingsOptions());
}

function displayHttproxyOptions()
{
    window.open("chrome://httproxy/content/httproxySettings.xul", "", "chrome");
}

window.addEventListener("load", httproxyInitListener, false);
window.addEventListener("DOMContentLoaded", httproxyContentLoadedListener, false);
window.addEventListener("pageshow", httproxyPageShowListener, false);
